@@ -24,7 +24,7 @@
 #include "pkg.h"
 #include "parse.h"
 
-#include "popt.h"
+#include <popt.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -36,16 +36,10 @@
 #undef STRICT
 #endif
 
-#ifdef G_OS_WIN32
-/* No hardcoded paths in the binary, thanks */
-#undef PKGLIBDIR
-/* It's OK to leak this, as PKGLIBDIR is invoked only once */
-#define PKG_CONFIG_PATH g_strconcat (g_win32_get_package_installation_directory (PACKAGE, NULL), "\\lib\\pkgconfig", NULL)
-#endif
-
 static int want_debug_spew = 0;
 static int want_verbose_errors = 0;
 static int want_stdout_errors = 0;
+char *pcsysrootdir = NULL;
 
 void
 debug_spew (const char *format, ...)
@@ -169,41 +163,49 @@ pkg_uninstalled (Package *pkg)
   return FALSE;
 }
 
+void
+print_hashtable_key (gpointer key,
+                     gpointer value,
+                     gpointer user_data)
+{
+  printf("%s\n", (gchar*)key);
+}
+
 int
 main (int argc, char **argv)
 {
-  int want_my_version = 0;
-  int want_version = 0;
-  int want_libs = 0;
-  int want_cflags = 0;
-  int want_l_libs = 0;
-  int want_L_libs = 0;
-  int want_other_libs = 0;
-  int want_I_cflags = 0;
-  int want_other_cflags = 0;
-  int want_list = 0;
-  int want_static_lib_list = ENABLE_INDIRECT_DEPS;
-  int want_short_errors = 0;
+  static int want_my_version = 0;
+  static int want_version = 0;
+  static int want_libs = 0;
+  static int want_cflags = 0;
+  static int want_l_libs = 0;
+  static int want_L_libs = 0;
+  static int want_other_libs = 0;
+  static int want_I_cflags = 0;
+  static int want_other_cflags = 0;
+  static int want_list = 0;
+  static int want_static_lib_list = ENABLE_INDIRECT_DEPS;
+  static int want_short_errors = 0;
+  static int want_uninstalled = 0;
+  static char *variable_name = NULL;
+  static int want_exists = 0;
+  static int want_provides = 0;
+  static int want_requires = 0;
+  static int want_requires_private = 0;
+  static char *required_atleast_version = NULL;
+  static char *required_exact_version = NULL;
+  static char *required_max_version = NULL;
+  static char *required_pkgconfig_version = NULL;
+  static int want_silence_errors = 0;
+  static int want_variable_list = 0;
   int result;
-  int want_uninstalled = 0;
-  char *variable_name = NULL;
-  int want_exists = 0;
-  char *required_atleast_version = NULL;
-  char *required_exact_version = NULL;
-  char *required_max_version = NULL;
-  char *required_pkgconfig_version = NULL;
-  int want_silence_errors = 0;
   GString *str;
   GSList *packages = NULL;
   char *search_path;
   char *pcbuilddir;
-  const char *pkglibdir;
-  char **search_dirs;
-  char **iter;
   gboolean need_newline;
-
+  FILE *log = NULL;
   const char *pkgname;
-  Package *pkg;
 
   poptContext opt_context;
 
@@ -234,11 +236,13 @@ main (int argc, char **argv)
     { "cflags-only-other", 0, POPT_ARG_NONE, &want_other_cflags, 0,
       "output cflags not covered by the cflags-only-I option"},
     { "variable", 0, POPT_ARG_STRING, &variable_name, 0,
-      "get the value of a variable", "VARIABLENAME" },
+      "get the value of variable named NAME", "NAME" },
     { "define-variable", 0, POPT_ARG_STRING, NULL, DEFINE_VARIABLE,
-      "set the value of a variable", "VARIABLENAME=VARIABLEVALUE" },
+      "set variable NAME to VALUE", "NAME=VALUE" },
     { "exists", 0, POPT_ARG_NONE, &want_exists, 0,
       "return 0 if the module(s) exist" },
+    { "print-variables", 0, POPT_ARG_NONE, &want_variable_list, 0,
+      "output list of variables defined by the module" },
     { "uninstalled", 0, POPT_ARG_NONE, &want_uninstalled, 0,
       "return 0 if the uninstalled version of one or more module(s) or their dependencies will be used" },
     { "atleast-version", 0, POPT_ARG_STRING, &required_atleast_version, 0,
@@ -252,11 +256,19 @@ main (int argc, char **argv)
     { "debug", 0, POPT_ARG_NONE, &want_debug_spew, 0,
       "show verbose debug information" },
     { "print-errors", 0, POPT_ARG_NONE, &want_verbose_errors, 0,
-      "show verbose information about missing or conflicting packages" },
+      "show verbose information about missing or conflicting packages,"
+      "default if --cflags or --libs given on the command line" },
     { "silence-errors", 0, POPT_ARG_NONE, &want_silence_errors, 0,
-      "show verbose information about missing or conflicting packages" },
+      "be silent about errors (default unless --cflags or --libs"
+      "given on the command line)" },
     { "errors-to-stdout", 0, POPT_ARG_NONE, &want_stdout_errors, 0,
       "print errors from --print-errors to stdout not stderr" },
+    { "print-provides", 0, POPT_ARG_NONE, &want_provides, 0,
+      "print which packages the package provides" },
+    { "print-requires", 0, POPT_ARG_NONE, &want_requires, 0,
+      "print which packages the package requires" },
+    { "print-requires-private", 0, POPT_ARG_NONE, &want_requires_private, 0,
+      "print which packages the package requires for static linking" },
 #ifdef G_OS_WIN32
     { "dont-define-prefix", 0, POPT_ARG_NONE, &dont_define_prefix, 0,
       "don't try to override the value of prefix for each .pc file found with "
@@ -281,71 +293,29 @@ main (int argc, char **argv)
       debug_spew ("PKG_CONFIG_DEBUG_SPEW variable enabling debug spew\n");
     }
 
+  search_path = getenv ("PKG_CONFIG_PATH");
+  if (search_path) 
+    {
+      add_search_dirs(search_path, G_SEARCHPATH_SEPARATOR_S);
+    }
   if (getenv("PKG_CONFIG_LIBDIR") != NULL) 
     {
       add_search_dirs(getenv("PKG_CONFIG_LIBDIR"), G_SEARCHPATH_SEPARATOR_S);
     }
   else
     {
-      search_path = getenv ("PKG_CONFIG_PATH");
-      if (search_path) 
-	{
-	  add_search_dirs(search_path, G_SEARCHPATH_SEPARATOR_S);
-	}
-      
       add_search_dirs(PKG_CONFIG_PC_PATH, G_SEARCHPATH_SEPARATOR_S);
     }
 
-#ifdef G_OS_WIN32
-  {
-    /* Add search directories from the Registry */
-
-    HKEY roots[] = { HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE };
-    gchar *root_names[] = { "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE" };
-    HKEY key;
-    int i;
-    gulong max_value_name_len, max_value_len;
-
-    for (i = 0; i < G_N_ELEMENTS (roots); i++)
-      {
-	key = NULL;
-	if (RegOpenKeyEx (roots[i], "Software\\" PACKAGE "\\PKG_CONFIG_PATH", 0,
-			  KEY_QUERY_VALUE, &key) == ERROR_SUCCESS &&
-	    RegQueryInfoKey (key, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-			     &max_value_name_len, &max_value_len,
-			     NULL, NULL) == ERROR_SUCCESS)
-	  {
-	    int index = 0;
-	    gchar *value_name = g_malloc (max_value_name_len + 1);
-	    gchar *value = g_malloc (max_value_len + 1);
-
-	    while (TRUE)
-	      {
-		gulong type;
-		gulong value_name_len = max_value_name_len + 1;
-		gulong value_len = max_value_len + 1;
-
-		if (RegEnumValue (key, index++, value_name, &value_name_len,
-				  NULL, &type,
-				  value, &value_len) != ERROR_SUCCESS)
-		  break;
-
-		if (type != REG_SZ)
-		  continue;
-
-		value_name[value_name_len] = '\0';
-		value[value_len] = '\0';
-		debug_spew ("Adding directory '%s' from %s\\Software\\"
-			    PACKAGE "\\PKG_CONFIG_PATH\\%s\n",
-			    value, root_names[i], value_name);
-		add_search_dir (value);
-	      }
-	  }
-	if (key != NULL)
-	  RegCloseKey (key);
-      }
-  }
-#endif
+  pcsysrootdir = getenv ("PKG_CONFIG_SYSROOT_DIR");
+  if (pcsysrootdir)
+    {
+      define_global_variable ("pc_sysrootdir", pcsysrootdir);
+    }
+  else
+    {
+      define_global_variable ("pc_sysrootdir", "/");
+    }
 
   pcbuilddir = getenv ("PKG_CONFIG_TOP_BUILD_DIR");
   if (pcbuilddir)
@@ -394,7 +364,8 @@ main (int argc, char **argv)
       want_other_libs ||
       want_I_cflags ||
       want_other_cflags ||
-      want_list)
+      want_list ||
+      want_variable_list)
     {
       debug_spew ("Error printing enabled by default due to use of --version, --libs, --cflags, --libs-only-l, --libs-only-L, --libs-only-other, --cflags-only-I, --cflags-only-other or --list. Value of --silence-errors: %d\n", want_silence_errors);
 
@@ -421,6 +392,20 @@ main (int argc, char **argv)
   else
     disable_private_libs();
 
+  /* honor Requires.private if any Cflags are requested or any static
+   * libs are requested */
+
+  if (want_I_cflags || want_other_cflags || want_cflags ||
+      want_requires_private ||
+      (want_static_lib_list && (want_libs || want_l_libs || want_L_libs)))
+    enable_requires_private();
+
+  /* ignore Requires if no Cflags or Libs are requested */
+
+  if (!want_I_cflags && !want_other_cflags && !want_cflags &&
+      !want_libs && !want_l_libs && !want_L_libs && !want_requires)
+    disable_requires();
+
   if (want_my_version)
     {
       printf ("%s\n", VERSION);
@@ -458,6 +443,17 @@ main (int argc, char **argv)
 
   g_strstrip (str->str);
 
+  if (getenv("PKG_CONFIG_LOG") != NULL)
+    {
+      log = fopen (getenv ("PKG_CONFIG_LOG"), "a");
+      if (log == NULL)
+	{
+	  fprintf (stderr, "Cannot open log file: %s\n",
+		   getenv ("PKG_CONFIG_LOG"));
+	  exit (1);
+	}
+    }
+
   {
     gboolean failed = FALSE;
     GSList *reqs;
@@ -478,6 +474,17 @@ main (int argc, char **argv)
         else
           req = get_package (ver->name);
 
+	if (log != NULL)
+	  {
+	    if (req == NULL)
+	      fprintf (log, "%s NOT-FOUND", ver->name);
+	    else
+	      fprintf (log, "%s %s %s", ver->name,
+		       comparison_to_str (ver->comparison),
+		       (ver->version == NULL) ? "(null)" : ver->version);
+	    fprintf (log, "\n");
+	  }
+
         if (req == NULL)
           {
             failed = TRUE;
@@ -508,10 +515,31 @@ main (int argc, char **argv)
         iter = g_slist_next (iter);
       }
 
+    if (log != NULL)
+      {
+	fclose (log);
+      }
+
     if (failed) {
       return 1;
     }
 
+  if (want_variable_list)
+    {
+      GSList *tmp;
+      tmp = packages;
+      while (tmp != NULL)
+        {
+          Package *pkg = tmp->data;
+          g_hash_table_foreach(pkg->vars,
+                               &print_hashtable_key,
+                               NULL);
+          tmp = g_slist_next (tmp);
+          if (tmp) printf ("\n");
+        }
+      need_newline = FALSE;
+    }
+
   }
 
   g_string_free (str, TRUE);
@@ -560,6 +588,74 @@ main (int argc, char **argv)
         }
     }
 
+ if (want_provides)
+   {
+     GSList *tmp;
+     tmp = packages;
+     while (tmp != NULL)
+       {
+         Package *pkg = tmp->data;
+         char *key;
+         key = pkg->key;
+         while (*key == '/')
+           key++;
+         if (strlen(key) > 0)
+           printf ("%s = %s\n", key, pkg->version);
+         tmp = g_slist_next (tmp);
+       }
+   }
+
+  if (want_requires)
+    {
+      GSList *pkgtmp;
+      for (pkgtmp = packages; pkgtmp != NULL; pkgtmp = g_slist_next (pkgtmp))
+        {
+          Package *pkg = pkgtmp->data;
+          GSList *reqtmp;
+
+          /* process Requires: */
+          for (reqtmp = pkg->requires; reqtmp != NULL; reqtmp = g_slist_next (reqtmp))
+            {
+              Package *deppkg = reqtmp->data;
+              RequiredVersion *req;
+              req = g_hash_table_lookup(pkg->required_versions, deppkg->key);
+              if ((req == NULL) || (req->comparison == ALWAYS_MATCH))
+                printf ("%s\n", deppkg->key);
+              else
+                printf ("%s %s %s\n", deppkg->key,
+                  comparison_to_str(req->comparison),
+                  req->version);
+            }
+        }
+    }
+  if (want_requires_private)
+    {
+      GSList *pkgtmp;
+      for (pkgtmp = packages; pkgtmp != NULL; pkgtmp = g_slist_next (pkgtmp))
+        {
+          Package *pkg = pkgtmp->data;
+          GSList *reqtmp;
+          /* process Requires.private: */
+          for (reqtmp = pkg->requires_private; reqtmp != NULL; reqtmp = g_slist_next (reqtmp))
+            {
+
+              Package *deppkg = reqtmp->data;
+              RequiredVersion *req;
+
+              if (g_slist_find (pkg->requires, reqtmp->data))
+                continue;
+
+              req = g_hash_table_lookup(pkg->required_versions, deppkg->key);
+              if ((req == NULL) || (req->comparison == ALWAYS_MATCH))
+                printf ("%s\n", deppkg->key);
+              else
+                printf ("%s %s %s\n", deppkg->key,
+                  comparison_to_str(req->comparison),
+                  req->version);
+            }
+        }
+    }
+  
   if (required_exact_version)
     {
       Package *pkg = packages->data;
@@ -651,7 +747,11 @@ main (int argc, char **argv)
     }
 
   if (need_newline)
+#ifdef G_OS_WIN32
+    printf ("\r\n");
+#else
     printf ("\n");
+#endif
 
   return 0;
 }
