@@ -1,4 +1,5 @@
 /* 
+ * Copyright (C) 2006-2011 Tollef Fog Heen <tfheen@err.no>
  * Copyright (C) 2001, 2002, 2005-2006 Red Hat Inc.
  * 
  * This program is free software; you can redistribute it and/or
@@ -27,7 +28,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include "popt.h"
+#include <popt.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
@@ -39,6 +40,12 @@ char *prefix_variable = "prefix";
 int msvc_syntax = FALSE;
 #endif
 
+#ifdef G_OS_WIN32
+#ifndef G_IS_DIR_SEPARATOR
+#define G_IS_DIR_SEPARATOR(c) ((c) == G_DIR_SEPARATOR || (c) == '/')
+#endif
+#endif
+
 /**
  * Read an entire line from a file into a buffer. Lines may
  * be delimited with '\n', '\r', '\n\r', or '\r\n'. The delimiter
@@ -558,8 +565,6 @@ parse_requires (Package *pkg, const char *str, const char *path)
     }
 
   g_slist_free (parsed);
-  
-  pkg->requires = g_slist_reverse (pkg->requires);
 }
 
 static void
@@ -607,15 +612,11 @@ parse_requires_private (Package *pkg, const char *str, const char *path)
     }
 
   g_slist_free (parsed);
-  
-  pkg->requires_private = g_slist_reverse (pkg->requires_private);
 }
 
 static void
 parse_conflicts (Package *pkg, const char *str, const char *path)
 {
-  GSList *parsed;
-  GSList *iter;
   char *trimmed;
   
   if (pkg->conflicts)
@@ -630,6 +631,34 @@ parse_conflicts (Package *pkg, const char *str, const char *path)
   g_free (trimmed);
 }
 
+static char *strdup_escape_shell(const char *s)
+{
+	size_t r_s = strlen(s)+10, c = 0;
+	char *r = g_malloc(r_s);
+	while (s[0]) {
+		if ((s[0] < '$') ||
+		    (s[0] > '$' && s[0] < '(') ||
+		    (s[0] > ')' && s[0] < '+') ||
+		    (s[0] > ':' && s[0] < '=') ||
+		    (s[0] > '=' && s[0] < '@') ||
+		    (s[0] > 'Z' && s[0] < '^') ||
+		    (s[0] == '`') ||
+		    (s[0] > 'z')) {
+			r[c] = '\\';
+			c++;
+		}
+		r[c] = *s;
+		c++;
+		if (c+2 >= r_s) {
+			r_s *= 2;
+			r = g_realloc(r, r_s);
+		}
+		s++;
+	}
+	r[c] = 0;
+	return r;
+}
+
 static void _do_parse_libs (Package *pkg, int argc, char **argv)
 {
   int i;
@@ -646,53 +675,35 @@ static void _do_parse_libs (Package *pkg, int argc, char **argv)
   i = 0;
   while (i < argc)
     {
-      char *arg = trim_string (argv[i]);
+      char *tmp = trim_string (argv[i]);
+      char *arg = strdup_escape_shell(tmp);
       char *p;
-      char *start;
-
-      start = arg;
-      p = start;      
+      p = arg;
+      g_free(tmp);
 
       if (p[0] == '-' &&
-          p[1] == 'l')
+          p[1] == 'l' &&
+	  /* -lib: is used by the C# compiler for libs; it's not an -l
+              flag. */
+	  (strncmp(p, "-lib:", 5) != 0))
         {
-          char *libname;          
-              
           p += 2;
           while (*p && isspace ((guchar)*p))
             ++p;
-              
-          start = p;
-          while (*p && !isspace ((guchar)*p))
-            ++p;
 
-          libname = g_strndup (start, p - start);
-          
           pkg->l_libs = g_slist_prepend (pkg->l_libs,
-                                         g_strconcat (l_flag, libname, lib_suffix, NULL));
+                                         g_strconcat (l_flag, p, lib_suffix, NULL));
 
-          g_free (libname);
         }
       else if (p[0] == '-' &&
                p[1] == 'L')
         {
-          char *libname;          
-          
           p += 2;
           while (*p && isspace ((guchar)*p))
             ++p;
-              
-          start = p;
-          while (*p && !isspace ((guchar)*p))
-            ++p;
-
-          libname = g_strndup (start, p - start);
-          
-          pkg->L_libs = g_slist_prepend (pkg->L_libs,
-					 g_strconcat (L_flag, libname, NULL));
-
-          g_free (libname);
-        }
+	  pkg->L_libs = g_slist_prepend (pkg->L_libs,
+					 g_strconcat (L_flag, p, NULL));
+	}
       else if (strcmp("-framework",p) == 0 && i+1 < argc)
         {
           /* Mac OS X has a -framework Foo which is really one option,
@@ -700,12 +711,14 @@ static void _do_parse_libs (Package *pkg, int argc, char **argv)
            * -framework Bar being changed into -framework Foo Bar
            * later
           */
-          gchar *framework = trim_string (argv[i+1]);
+          gchar *framework, *tmp = trim_string (argv[i+1]);
 
+	  framework = strdup_escape_shell(tmp);
           pkg->other_libs = g_slist_prepend (pkg->other_libs,
                                              g_strconcat(arg, " ", framework, NULL));
           i++;
           g_free(framework);
+          g_free(tmp);
         }
       else
         {
@@ -715,14 +728,10 @@ static void _do_parse_libs (Package *pkg, int argc, char **argv)
         }
 
       g_free (arg);
-      
+
       ++i;
     }
 
-  pkg->l_libs = g_slist_reverse (pkg->l_libs);
-  pkg->L_libs = g_slist_reverse (pkg->L_libs);
-  pkg->other_libs = g_slist_reverse (pkg->other_libs);
-  
 }
 
 
@@ -733,7 +742,7 @@ parse_libs (Package *pkg, const char *str, const char *path)
   
   char *trimmed;
   char **argv = NULL;
-  int argc;
+  int argc = 0;
   int result;
   
   if (pkg->libs_num > 0)
@@ -745,14 +754,17 @@ parse_libs (Package *pkg, const char *str, const char *path)
   
   trimmed = trim_and_sub (pkg, str, path);
 
-  result = poptParseArgvString (trimmed, &argc, &argv);
-
-  if (result < 0)
+  if (trimmed && *trimmed)
     {
-      verbose_error ("Couldn't parse Libs field into an argument vector: %s\n",
-               poptStrerror (result));
+      result = poptParseArgvString (trimmed, &argc, &argv);
 
-      exit (1);
+      if (result < 0)
+        {
+          verbose_error ("Couldn't parse Libs field into an argument vector: %s\n",
+                         poptStrerror (result));
+
+          exit (1);
+        }
     }
 
   _do_parse_libs(pkg, argc, argv);
@@ -779,7 +791,7 @@ parse_libs_private (Package *pkg, const char *str, const char *path)
   
   char *trimmed;
   char **argv = NULL;
-  int argc;
+  int argc = 0;
   int result;
   
   if (pkg->libs_private_num > 0)
@@ -791,14 +803,17 @@ parse_libs_private (Package *pkg, const char *str, const char *path)
   
   trimmed = trim_and_sub (pkg, str, path);
 
-  result = poptParseArgvString (trimmed, &argc, &argv);
-
-  if (result < 0)
+  if (trimmed && *trimmed)
     {
-      verbose_error ("Couldn't parse Libs.private field into an argument vector: %s\n",
-               poptStrerror (result));
+      result = poptParseArgvString (trimmed, &argc, &argv);
 
-      exit (1);
+      if (result < 0)
+        {
+          verbose_error ("Couldn't parse Libs.private field into an argument vector: %s\n",
+                         poptStrerror (result));
+
+          exit (1);
+        }
     }
 
   _do_parse_libs(pkg, argc, argv);
@@ -816,7 +831,7 @@ parse_cflags (Package *pkg, const char *str, const char *path)
   
   char *trimmed;
   char **argv = NULL;
-  int argc;
+  int argc = 0;
   int result;
   int i;
   
@@ -829,52 +844,50 @@ parse_cflags (Package *pkg, const char *str, const char *path)
   
   trimmed = trim_and_sub (pkg, str, path);
 
-  result = poptParseArgvString (trimmed, &argc, &argv);
-
-  if (result < 0)
+  if (trimmed && *trimmed)
     {
-      verbose_error ("Couldn't parse Cflags field into an argument vector: %s\n",
-                     poptStrerror (result));
+      result = poptParseArgvString (trimmed, &argc, &argv);
 
-      exit (1);
+      if (result < 0)
+        {
+          verbose_error ("Couldn't parse Cflags field into an argument vector: %s\n",
+                         poptStrerror (result));
+
+          exit (1);
+        }
     }
 
   i = 0;
   while (i < argc)
     {
-      char *arg = trim_string (argv[i]);
-      char *p;
-      char *start;
-
-      start = arg;
-      p = start;      
+      char *tmp = trim_string (argv[i]);
+      char *arg = strdup_escape_shell(tmp);
+      char *p = arg;
+      g_free(tmp);
 
       if (p[0] == '-' &&
           p[1] == 'I')
         {
-          char *libname;          
-              
           p += 2;
           while (*p && isspace ((guchar)*p))
             ++p;
-              
-          start = p;
-          while (*p && !isspace ((guchar)*p))
-            ++p;
 
-          libname = g_strndup (start, p - start);
-          
           pkg->I_cflags = g_slist_prepend (pkg->I_cflags,
-                                           g_strconcat ("-I", libname, NULL));
+                                           g_strconcat ("-I", p, NULL));
 
-          g_free (libname);
-        }
-      else
-        {
+        } else {
           if (*arg != '\0')
             pkg->other_cflags = g_slist_prepend (pkg->other_cflags,
                                                  g_strdup (arg));
-        }
+	  if (strcmp("-idirafter", arg) == 0) {
+	      char *n;
+
+	      tmp = trim_string(argv[++i]);
+	      n = strdup_escape_shell(tmp);
+	      pkg->other_cflags = g_slist_prepend(pkg->other_cflags, n);
+	      g_free(tmp);
+	  }
+      }
 
       g_free (arg);
       
@@ -883,9 +896,6 @@ parse_cflags (Package *pkg, const char *str, const char *path)
 
   g_free (argv);
   g_free (trimmed);
-
-  pkg->I_cflags = g_slist_reverse (pkg->I_cflags);
-  pkg->other_cflags = g_slist_reverse (pkg->other_cflags);
 }
 
 static void
@@ -901,8 +911,28 @@ parse_url (Package *pkg, const char *str, const char *path)
   pkg->url = trim_and_sub (pkg, str, path);
 }
 
+#ifdef G_OS_WIN32
+static char *orig_prefix = NULL;
+
+static int
+pathnamecmp (const char *a,
+	     const char *b)
+{
+  while (*a && *b &&
+	 ((G_IS_DIR_SEPARATOR (*a) && G_IS_DIR_SEPARATOR (*b)) ||
+	  g_ascii_toupper (*a) == g_ascii_toupper (*b)))
+    {
+      a++;
+      b++;
+    }
+  return g_ascii_toupper (*a) - g_ascii_toupper (*b);
+}
+#endif
+
 static void
-parse_line (Package *pkg, const char *untrimmed, const char *path, gboolean ignore_requires, gboolean ignore_private_libs)
+parse_line (Package *pkg, const char *untrimmed, const char *path,
+	    gboolean ignore_requires, gboolean ignore_private_libs,
+	    gboolean ignore_requires_private)
 {
   char *str;
   char *p;
@@ -946,7 +976,10 @@ parse_line (Package *pkg, const char *untrimmed, const char *path, gboolean igno
       else if (strcmp (tag, "Version") == 0)
         parse_version (pkg, p, path);
       else if (strcmp (tag, "Requires.private") == 0)
-	parse_requires_private (pkg, p, path);
+	{
+	  if (!ignore_requires_private)
+	    parse_requires_private (pkg, p, path);
+	}
       else if (strcmp (tag, "Requires") == 0)
 	{
           if (ignore_requires == FALSE)
@@ -1000,29 +1033,38 @@ parse_line (Package *pkg, const char *untrimmed, const char *path, gboolean igno
 	  gchar *prefix = pkg->pcfiledir;
 	  const int prefix_len = strlen (prefix);
 	  const char *const lib_pkgconfig = "\\lib\\pkgconfig";
+	  const char *const share_pkgconfig = "\\share\\pkgconfig";
 	  const int lib_pkgconfig_len = strlen (lib_pkgconfig);
+	  const int share_pkgconfig_len = strlen (share_pkgconfig);
 
-	  if (strlen (prefix) > lib_pkgconfig_len &&
-	      g_ascii_strcasecmp (prefix + prefix_len - lib_pkgconfig_len,
-				  lib_pkgconfig) == 0)
+	  if ((strlen (prefix) > lib_pkgconfig_len &&
+	       pathnamecmp (prefix + prefix_len - lib_pkgconfig_len, lib_pkgconfig) == 0) ||
+	      (strlen (prefix) > share_pkgconfig_len &&
+	       pathnamecmp (prefix + prefix_len - share_pkgconfig_len, share_pkgconfig) == 0))
 	    {
-	      /* It ends in lib\pkgconfig. Good. */
+	      /* It ends in lib\pkgconfig or share\pkgconfig. Good. */
 	      
-	      gchar *p;
+	      gchar *q;
 	      
+	      orig_prefix = g_strdup (p);
+
 	      prefix = g_strdup (prefix);
-	      prefix[prefix_len - lib_pkgconfig_len] = '\0';
+	      if (strlen (prefix) > lib_pkgconfig_len &&
+		  pathnamecmp (prefix + prefix_len - lib_pkgconfig_len, lib_pkgconfig) == 0)
+		prefix[prefix_len - lib_pkgconfig_len] = '\0';
+	      else
+		prefix[prefix_len - share_pkgconfig_len] = '\0';
 	      
 	      /* Turn backslashes into slashes or
 	       * poptParseArgvString() will eat them when ${prefix}
 	       * has been expanded in parse_libs().
 	       */
-	      p = prefix;
-	      while (*p)
+	      q = prefix;
+	      while (*q)
 		{
-		  if (*p == '\\')
-		    *p = '/';
-		  p++;
+		  if (*q == '\\')
+		    *q = '/';
+		  q++;
 		}
 	      varname = g_strdup (tag);
 	      debug_spew (" Variable declaration, '%s' overridden with '%s'\n",
@@ -1031,6 +1073,16 @@ parse_line (Package *pkg, const char *untrimmed, const char *path, gboolean igno
 	      goto cleanup;
 	    }
 	}
+      else if (!dont_define_prefix &&
+	       orig_prefix != NULL &&
+	       strncmp (p, orig_prefix, strlen (orig_prefix)) == 0 &&
+	       G_IS_DIR_SEPARATOR (p[strlen (orig_prefix)]))
+	{
+	  char *oldstr = str;
+
+	  p = str = g_strconcat (g_hash_table_lookup (pkg->vars, prefix_variable), p + strlen (orig_prefix), NULL);
+	  g_free (oldstr);
+	}
 #endif
 
       if (g_hash_table_lookup (pkg->vars, tag))
@@ -1056,7 +1108,9 @@ parse_line (Package *pkg, const char *untrimmed, const char *path, gboolean igno
 }
 
 Package*
-parse_package_file (const char *path, gboolean ignore_requires, gboolean ignore_private_libs)
+parse_package_file (const char *path, gboolean ignore_requires,
+		    gboolean ignore_private_libs,
+		    gboolean ignore_requires_private)
 {
   FILE *f;
   Package *pkg;
@@ -1093,7 +1147,8 @@ parse_package_file (const char *path, gboolean ignore_requires, gboolean ignore_
     {
       one_line = TRUE;
       
-      parse_line (pkg, str->str, path, ignore_requires, ignore_private_libs);
+      parse_line (pkg, str->str, path, ignore_requires, ignore_private_libs,
+		  ignore_requires_private);
 
       g_string_truncate (str, 0);
     }
@@ -1105,393 +1160,19 @@ parse_package_file (const char *path, gboolean ignore_requires, gboolean ignore_
   fclose(f);
 
   /* make ->requires_private include a copy of the public requires too */
-  pkg->requires_private = g_slist_concat(pkg->requires_private,
-					 g_slist_copy (pkg->requires));
+  pkg->requires_private = g_slist_concat(g_slist_copy (pkg->requires),
+					 pkg->requires_private);
+  
+  pkg->requires = g_slist_reverse (pkg->requires);
+  
+  pkg->requires_private = g_slist_reverse (pkg->requires_private);
 
+  pkg->I_cflags = g_slist_reverse (pkg->I_cflags);
+  pkg->other_cflags = g_slist_reverse (pkg->other_cflags);
+
+  pkg->l_libs = g_slist_reverse (pkg->l_libs);
+  pkg->L_libs = g_slist_reverse (pkg->L_libs);
+  pkg->other_libs = g_slist_reverse (pkg->other_libs);
+  
   return pkg;
 }
-
-static char *
-backticks (const char *command)
-{
-  FILE *f;
-  char buf[4096];
-  size_t len;
-  int status;
-  
-  f = popen (command, "r");
-
-  if (f == NULL)
-    return NULL;
-  
-  len = fread (buf, 1, 4090, f);
-
-  if (ferror (f))
-    {
-      pclose (f);
-      return NULL;
-    }
-  
-  buf[len] = '\0';
-
-  status = pclose (f);
-
-  return g_strdup (buf);
-}
-
-static gboolean
-try_command (const char *command)
-{
-  int status;
-  char *munged;
-
-#ifdef G_OS_WIN32
-  munged = g_strdup_printf ("%s > NUL", command);
-#else
-  munged = g_strdup_printf ("%s > /dev/null 2>&1", command);
-#endif
-  
-  status = system (munged);
-
-  g_free (munged);
-  
-#ifdef G_OS_WIN32
-  return status == 0;
-#else
-  return WIFEXITED(status) && (WEXITSTATUS(status) == 0);
-#endif
-}
-
-Package *
-get_compat_package (const char *name)
-{
-#ifdef G_OS_WIN32
-  /* There has never been any of these legacy *-config scripts on
-   * Windows as far as I know. No use trying to execute them, will
-   * only confuse users to see the "blabla is not recognized as an
-   * internal or external command, operable program or batch file"
-   * messages.
-   */
-  return NULL;
-#else
-
-  Package *pkg;
-
-  if (name_ends_in_uninstalled (name))
-    debug_spew ("Suspiciously looking for compat package for -uninstalled: %s\n", name);
-  
-  debug_spew ("Looking for '%s' using legacy -config scripts\n", name);
-  
-  pkg = g_new0 (Package, 1);
-
-  pkg->path_position = G_MAXINT;
-  
-  if (strcmp (name, "glib") == 0)
-    {
-      char *output;
-
-      debug_spew ("Calling glib-config\n");
-      
-      pkg->version = backticks ("glib-config --version");
-      if (pkg->version == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-      
-      pkg->name = g_strdup ("GLib");
-      pkg->key = g_strdup ("glib");
-      pkg->description = g_strdup ("C Utility Library");
-
-      output = backticks ("glib-config --libs");
-      parse_libs (pkg, output, "glib-config");
-      g_free (output);
-
-      output = backticks ("glib-config --cflags");
-      parse_cflags (pkg, output, "glib-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else if (strcmp (name, "gtk+") == 0)
-    {
-      char *output;
-
-      debug_spew ("Calling gtk-config\n");
-      
-      pkg->version = backticks ("gtk-config --version");
-      if (pkg->version == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-      
-      pkg->name = g_strdup ("GTK+");
-      pkg->key = g_strdup ("gtk+");
-      pkg->description = g_strdup ("GIMP Tool Kit");
-
-      output = backticks ("gtk-config --libs");
-      parse_libs (pkg, output, "gtk-config");
-      g_free (output);
-
-      output = backticks ("gtk-config --cflags");
-      parse_cflags (pkg, output, "gtk-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else if (strcmp (name, "libgnomevfs") == 0)
-    {
-      char *output;
-
-      debug_spew ("Calling gnome-vfs-config\n");
-      
-      pkg->version = backticks ("gnome-vfs-config --version");
-      if (pkg->version == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-      
-      pkg->name = g_strdup ("GNOME VFS");
-      pkg->key = g_strdup ("libgnomevfs");
-      pkg->description = g_strdup ("GNOME Virtual File System");
-
-      output = backticks ("gnome-vfs-config --libs");
-      parse_libs (pkg, output, "gnome-vfs-config");
-      g_free (output);
-
-      output = backticks ("gnome-vfs-config --cflags");
-      parse_cflags (pkg, output, "gnome-vfs-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else if (strcmp (name, "imlib") == 0)
-    {
-      char *output;
-
-      debug_spew ("Calling imlib-config\n");
-      
-      pkg->version = backticks ("imlib-config --version");
-      if (pkg->version == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-      
-      pkg->name = g_strdup ("Imlib");
-      pkg->key = g_strdup ("imlib");
-      pkg->description = g_strdup ("Imlib image loading library");
-
-      output = backticks ("imlib-config --libs-gdk");
-      parse_libs (pkg, output, "imlib-config");
-      g_free (output);
-
-      output = backticks ("imlib-config --cflags-gdk");
-      parse_cflags (pkg, output, "imlib-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else if (strcmp (name, "orbit-client") == 0)
-    {
-      char *output;
-      char *p;
-
-      debug_spew ("Calling orbit-config\n");
-      
-      output = backticks ("orbit-config --version");
-      
-      if (output == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-
-      p = output;
-
-      while (*p && isspace ((guchar)*p))
-        ++p;
-
-      if (*p == '\0')
-        {
-          /* empty output */
-          g_free (output);
-          g_free (pkg);
-          return NULL;
-        }
-
-      /* only heuristic; find a number or . */
-      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
-        ++p;      
-
-      pkg->version = g_strdup (p);
-
-      g_free (output);
-      
-      pkg->name = g_strdup ("ORBit Client");
-      pkg->key = g_strdup ("orbit-client");
-      pkg->description = g_strdup ("ORBit Client Libraries");
-
-      output = backticks ("orbit-config --libs client");
-      parse_libs (pkg, output, "orbit-config");
-      g_free (output);
-
-      output = backticks ("orbit-config --cflags client");
-      parse_cflags (pkg, output, "orbit-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else if (strcmp (name, "orbit-server") == 0)
-    {
-      char *output;
-      char *p;
-
-      debug_spew ("Calling orbit-config\n");
-      
-      output = backticks ("orbit-config --version");
-      
-      if (output == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-
-      p = output;
-
-      while (*p && isspace ((guchar)*p))
-        ++p;
-
-      if (*p == '\0')
-        {
-          /* empty output */
-          g_free (output);
-          g_free (pkg);
-          return NULL;
-        }
-
-      /* only heuristic; find a number or . */
-      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
-        ++p;      
-
-      pkg->version = g_strdup (p);
-
-      g_free (output);
-      
-      pkg->name = g_strdup ("ORBit Server");
-      pkg->key = g_strdup ("orbit-server");
-      pkg->description = g_strdup ("ORBit Server Libraries");
-
-      output = backticks ("orbit-config --libs server");
-      parse_libs (pkg, output, "orbit-config");
-      g_free (output);
-
-      output = backticks ("orbit-config --cflags server");
-      parse_cflags (pkg, output, "orbit-config");
-      g_free (output);
-
-      return pkg;
-    }
-  else
-    {
-      /* Check for the module in gnome-config */
-      char *output;
-      char *p;
-      char *command;
-
-      debug_spew ("Calling gnome-config\n");
-      
-      /* Annoyingly, --modversion doesn't return a failure
-       * code if the lib is unknown, so we have to use --libs
-       * for that.
-       */
-      
-      command = g_strdup_printf ("gnome-config --libs %s",
-                                 name);
-      
-      if (!try_command (command))
-        {
-          g_free (command);
-          g_free (pkg);
-          return NULL;
-        }
-      else
-        g_free (command);
-      
-      command = g_strdup_printf ("gnome-config --modversion %s",
-                                 name);
-      
-      output = backticks (command);
-      g_free (command);
-      if (output == NULL)
-        {
-          g_free (pkg);
-          return NULL;
-        }
-      
-      /* Unknown modules give "Unknown library `foo'" from gnome-config
-       * (but on stderr so this is useless, nevermind)
-       */
-      if (strstr (output, "Unknown") || *output == '\0')
-        {
-          g_free (output);
-          g_free (pkg);
-          return NULL;
-        }
-
-      /* gnome-config --modversion gnomeui outputs e.g. "gnome-libs-1.2.4"
-       * or libglade-0.12
-       */
-      p = output;
-
-      while (*p && isspace ((guchar)*p))
-        ++p;
-
-      if (*p == '\0')
-        {
-          /* empty output */
-          g_free (output);
-          g_free (pkg);
-          return NULL;
-        }
-
-      /* only heuristic; find a number or . */
-      while (*p && ! (isdigit ((guchar)*p) || *p == '.'))
-        ++p;      
-
-      pkg->version = g_strdup (p);
-
-      g_free (output);
-      
-      /* Strip newline */
-      p = pkg->version;
-      while (*p)
-        {
-          if (*p == '\n')
-            *p = '\0';
-
-          ++p;
-        }
-      
-      pkg->name = g_strdup (name);
-      pkg->key = g_strdup (name);
-      pkg->description = g_strdup ("No description");
-
-      command = g_strdup_printf ("gnome-config --libs %s", name);
-      output = backticks (command);
-      g_free (command);
-      parse_libs (pkg, output, "gnome-config");
-      g_free (output);
-
-      command = g_strdup_printf ("gnome-config --cflags %s", name);
-      output = backticks (command);
-      g_free (command);
-      parse_cflags (pkg, output, "gnome-config");
-      g_free (output);
-
-      return pkg;
-    }
-#endif
-}
