@@ -33,6 +33,10 @@
 # endif
 #endif
 
+#ifdef HAVE_MALLOC_H
+# include <malloc.h>
+#endif
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <string.h>
@@ -55,17 +59,9 @@ static int scanned_dir_count = 0;
 
 gboolean disable_uninstalled = FALSE;
 gboolean ignore_requires = FALSE;
+gboolean ignore_requires_private = TRUE;
 gboolean ignore_private_libs = TRUE;
 
-static Package pkg_config_package = {
-  .key = PACKAGE,
-  .name = PACKAGE,
-  .version = VERSION, /* .version */
-  .description = "returns metainformation about installed libraries",
-  .url = "http://pkg-config.freedesktop.org",
-  0 /* keep the rest as null */
-};
-
 void
 add_search_dir (const char *path)
 {
@@ -207,13 +203,17 @@ scan_dir (const char *dirname)
               filename[dirnamelen] = G_DIR_SEPARATOR;
               strcpy (filename + dirnamelen + 1, dent->d_name);
               
-              g_hash_table_insert (locations, pkgname, filename);
-              g_hash_table_insert (path_positions, pkgname,
-                                   GINT_TO_POINTER (scanned_dir_count));
-              
-              debug_spew ("Will find package '%s' in file '%s'\n",
-                          pkgname, filename);
-            }
+	      if (g_file_test(filename, G_FILE_TEST_IS_REGULAR) == TRUE) {
+		  g_hash_table_insert (locations, pkgname, filename);
+		  g_hash_table_insert (path_positions, pkgname,
+				       GINT_TO_POINTER (scanned_dir_count));
+		  debug_spew ("Will find package '%s' in file '%s'\n",
+			      pkgname, filename);
+	      } else {
+		  debug_spew ("Ignoring '%s' while looking for '%s'; not a "
+			      "regular file.\n", pkgname, filename);
+	      }
+	    }
         }
       else
         {
@@ -236,7 +236,11 @@ add_virtual_pkgconfig_package (void)
   pkg->name = g_strdup ("pkg-config");
   pkg->description = g_strdup ("pkg-config is a system for managing "
 			       "compile/link flags for libraries");
-  pkg->url = g_strdup ("http://www.freedesktop.org/software/pkgconfig/");
+  pkg->url = g_strdup ("http://pkg-config.freedesktop.org/");
+
+  if (pkg->vars == NULL)
+    pkg->vars = g_hash_table_new (g_str_hash, g_str_equal);
+  g_hash_table_insert (pkg->vars, "pc_path", PKG_CONFIG_PC_PATH);
 
   debug_spew ("Adding virtual 'pkg-config' package to list of known packages\n");
   g_hash_table_insert (packages, pkg->key, pkg);
@@ -263,30 +267,12 @@ package_init ()
     }
 }
 
-static gboolean
-file_readable (const char *path)
-{
-  FILE *f = fopen (path, "r");
-
-  if (f != NULL)
-    {
-      fclose (f);
-      return TRUE;
-    }
-  else
-    return FALSE;
-}
-
-
 static Package *
-internal_get_package (const char *name, gboolean warn, gboolean check_compat)
+internal_get_package (const char *name, gboolean warn)
 {
   Package *pkg = NULL;
   const char *location;
   
-  if (strcmp(PACKAGE, name) == 0)
-    return &pkg_config_package;
-
   pkg = g_hash_table_lookup (packages, name);
 
   if (pkg)
@@ -310,7 +296,7 @@ internal_get_package (const char *name, gboolean warn, gboolean check_compat)
 
           un = g_strconcat (name, "-uninstalled", NULL);
 
-          pkg = internal_get_package (un, FALSE, FALSE);
+          pkg = internal_get_package (un, FALSE);
 
           g_free (un);
           
@@ -324,19 +310,6 @@ internal_get_package (const char *name, gboolean warn, gboolean check_compat)
       location = g_hash_table_lookup (locations, name);
     }
   
-  if (location == NULL && check_compat)
-    {
-      pkg = get_compat_package (name);
-
-      if (pkg)
-        {
-          debug_spew ("Returning values for '%s' from a legacy -config script\n",
-                      name);
-          
-          return pkg;
-        }
-    }
-      
   if (location == NULL)
     {
       if (warn)
@@ -349,7 +322,8 @@ internal_get_package (const char *name, gboolean warn, gboolean check_compat)
     }
 
   debug_spew ("Reading '%s' from file '%s'\n", name, location);
-  pkg = parse_package_file (location, ignore_requires, ignore_private_libs);
+  pkg = parse_package_file (location, ignore_requires, ignore_private_libs, 
+			    ignore_requires_private);
   
   if (pkg == NULL)
     {
@@ -396,13 +370,13 @@ internal_get_package (const char *name, gboolean warn, gboolean check_compat)
 Package *
 get_package (const char *name)
 {
-  return internal_get_package (name, TRUE, TRUE);
+  return internal_get_package (name, TRUE);
 }
 
 Package *
 get_package_quiet (const char *name)
 {
-  return internal_get_package (name, FALSE, TRUE);
+  return internal_get_package (name, FALSE);
 }
 
 static GSList*
@@ -481,13 +455,22 @@ string_list_to_string (GSList *list)
   char *retval;
   
   tmp = list;
-  while (tmp != NULL)
-    {
-      g_string_append (str, tmp->data);
-      g_string_append_c (str, ' ');
-      
-      tmp = g_slist_next (tmp);
+  while (tmp != NULL) {
+    char *tmpstr = (char*) tmp->data;
+    if (pcsysrootdir != NULL &&
+	tmpstr[0] == '-' &&
+	(tmpstr[1] == 'I' ||
+	 tmpstr[1] == 'L')) {
+      g_string_append_c (str, '-');
+      g_string_append_c (str, tmpstr[1]);
+      g_string_append (str, pcsysrootdir);
+      g_string_append (str, tmpstr+2);
+    } else {
+      g_string_append (str, tmpstr);
     }
+    g_string_append_c (str, ' ');
+    tmp = g_slist_next (tmp);
+  }
 
   retval = str->str;
   g_string_free (str, FALSE);
@@ -577,22 +560,6 @@ spew_package_list (const char *name,
   debug_spew ("\n");
 }
 
-static void
-spew_string_list (const char *name,
-                  GSList     *list)
-{
-  GSList *tmp;
-
-  debug_spew (" %s: ", name);
-  
-  tmp = list;
-  while (tmp != NULL)
-    {
-      debug_spew (" %s ", tmp->data);
-      tmp = tmp->next;
-    }
-  debug_spew ("\n");
-}
 
 static GSList*
 packages_sort_by_path_position (GSList *list)
@@ -705,24 +672,6 @@ fill_list (GSList *packages, GetListFunc func,
   g_slist_free (expanded);
 }
 
-static gint
-compare_req_version_names (gconstpointer a, gconstpointer b)
-{
-  const RequiredVersion *ver_a = a;
-  const RequiredVersion *ver_b = b;
-
-  return strcmp (ver_a->name, ver_b->name);
-}
-
-static gint
-compare_package_keys (gconstpointer a, gconstpointer b)
-{
-  const Package *pkg_a = a;
-  const Package *pkg_b = b;
-
-  return strcmp (pkg_a->key, pkg_b->key);
-}
-
 static GSList *
 add_env_variable_to_list (GSList *list, const gchar *env)
 {
@@ -750,7 +699,7 @@ verify_package (Package *pkg)
   GSList *conflicts_iter;
   GSList *system_dir_iter = NULL;
   int count;
-  gchar *c_include_path;
+  const gchar *search_path;
 
   /* Be sure we have the required fields */
 
@@ -821,7 +770,7 @@ verify_package (Package *pkg)
    */
   
   recursive_fill_list (pkg, get_requires_private, &requires);
-  recursive_fill_list (pkg, get_conflicts, &conflicts);
+  conflicts = get_conflicts (pkg);
 
   requires_iter = requires;
   while (requires_iter != NULL)
@@ -834,16 +783,17 @@ verify_package (Package *pkg)
         {
           RequiredVersion *ver = conflicts_iter->data;
 
-          if (version_test (ver->comparison,
-                            req->version,
-                            ver->version))
+	  if (strcmp (ver->name, req->key) == 0 &&
+	      version_test (ver->comparison,
+			    req->version,
+			    ver->version))
             {
               verbose_error ("Version %s of %s creates a conflict.\n"
                              "(%s %s %s conflicts with %s %s)\n",
                              req->version, req->name,
                              ver->name,
                              comparison_to_str (ver->comparison),
-                             ver->version,
+                             ver->version ? ver->version : "(any)",
                              ver->owner->name,
                              ver->owner->version);
 
@@ -857,25 +807,30 @@ verify_package (Package *pkg)
     }
   
   g_slist_free (requires);
-  g_slist_free (conflicts);
 
   /* We make a list of system directories that gcc expects so we can remove
    * them.
    */
-#ifndef G_OS_WIN32
-  system_directories = g_slist_append (NULL, g_strdup ("/usr/include"));
-#endif
 
-  c_include_path = g_getenv ("C_INCLUDE_PATH");
-  if (c_include_path != NULL)
+  search_path = g_getenv ("PKG_CONFIG_SYSTEM_INCLUDE_PATH");
+
+  if (search_path == NULL)
     {
-      system_directories = add_env_variable_to_list (system_directories, c_include_path);
+      search_path = PKG_CONFIG_SYSTEM_INCLUDE_PATH;
     }
-  
-  c_include_path = g_getenv ("CPLUS_INCLUDE_PATH");
-  if (c_include_path != NULL)
+
+  system_directories = add_env_variable_to_list (system_directories, search_path);
+
+  search_path = g_getenv ("C_INCLUDE_PATH");
+  if (search_path != NULL)
     {
-      system_directories = add_env_variable_to_list (system_directories, c_include_path);
+      system_directories = add_env_variable_to_list (system_directories, search_path);
+    }
+
+  search_path = g_getenv ("CPLUS_INCLUDE_PATH");
+  if (search_path != NULL)
+    {
+      system_directories = add_env_variable_to_list (system_directories, search_path);
     }
 
   count = 0;
@@ -928,31 +883,52 @@ verify_package (Package *pkg)
   g_slist_foreach (system_directories, (GFunc) g_free, NULL);
   g_slist_free (system_directories);
 
-#ifdef PREFER_LIB64
-#define SYSTEM_LIBDIR "/usr/lib64"
-#else
-#define SYSTEM_LIBDIR "/usr/lib"
-#endif
+  system_directories = NULL;
+
+  search_path = g_getenv ("PKG_CONFIG_SYSTEM_LIBRARY_PATH");
+
+  if (search_path == NULL)
+    {
+      search_path = PKG_CONFIG_SYSTEM_LIBRARY_PATH;
+    }
+
+  system_directories = add_env_variable_to_list (system_directories, search_path);
+
   count = 0;
   iter = pkg->L_libs;
   while (iter != NULL)
     {
-      if (strcmp (iter->data, "-L" SYSTEM_LIBDIR) == 0 ||
-          strcmp (iter->data, "-L " SYSTEM_LIBDIR) == 0)
-        {
-          debug_spew ("Package %s has -L" SYSTEM_LIBDIR " in Libs\n",
-                      pkg->name);
-          if (g_getenv ("PKG_CONFIG_ALLOW_SYSTEM_LIBS") == NULL)
-            {              
-              iter->data = NULL;
-              ++count;
-              debug_spew ("Removing -L" SYSTEM_LIBDIR " from libs for %s\n", pkg->key);
-            }
-        }
+      GSList *system_dir_iter = system_directories;
 
+      while (system_dir_iter != NULL)
+        {
+          gboolean is_system = FALSE;
+          const char *linker_arg = iter->data;
+          const char *system_libpath = system_dir_iter->data;
+
+          if (strncmp (linker_arg, "-L ", 3) == 0 &&
+              strcmp (linker_arg + 3, system_libpath) == 0)
+            is_system = TRUE;
+          else if (strncmp (linker_arg, "-L", 2) == 0 &&
+              strcmp (linker_arg + 2, system_libpath) == 0)
+            is_system = TRUE;
+          if (is_system)
+            {
+              debug_spew ("Package %s has -L %s in Libs\n",
+                          pkg->name, system_libpath);
+              if (g_getenv ("PKG_CONFIG_ALLOW_SYSTEM_LIBS") == NULL)
+                {
+                  iter->data = NULL;
+                  ++count;
+                  debug_spew ("Removing -L %s from libs for %s\n", system_libpath, pkg->key);
+                  break;
+                }
+            }
+          system_dir_iter = system_dir_iter->next;
+        }
       iter = iter->next;
     }
-#undef SYSTEM_LIBDIR
+  g_slist_free (system_directories);
 
   while (count)
     {
@@ -1009,7 +985,6 @@ static char*
 get_multi_merged (GSList *pkgs, GetListFunc func, gboolean in_path_order,
 		  gboolean include_private)
 {
-  GSList *tmp;
   GSList *dups_list = NULL;
   GSList *list;
   char *retval;
@@ -1031,7 +1006,6 @@ static char*
 get_multi_merged_from_back (GSList *pkgs, GetListFunc func,
 			    gboolean in_path_order, gboolean include_private)
 {
-  GSList *tmp;
   GSList *dups_list = NULL;
   GSList *list;
   char *retval;
@@ -1316,6 +1290,9 @@ static int rpmvercmp(const char * a, const char * b) {
 	while (*one && !isalnum((guchar)*one)) one++;
 	while (*two && !isalnum((guchar)*two)) two++;
 
+	/* If we ran to the end of either, we are finished with the loop */
+	if (!(*one && *two)) break;
+
 	str1 = one;
 	str2 = two;
 
@@ -1342,7 +1319,8 @@ static int rpmvercmp(const char * a, const char * b) {
 	/* take care of the case where the two version segments are */
 	/* different types: one numeric and one alpha */
 	if (one == str1) return -1;	/* arbitrary */
-	if (two == str2) return -1;
+	/* XXX See patch #60884 (and details) from bugzilla #50977. */
+	if (two == str2) return (isnum ? 1 : -1);
 
 	if (isnum) {
 	    /* this used to be done by converting the digit segments */
@@ -1501,8 +1479,9 @@ void
 print_package_list (void)
 {
   int mlen = 0;
-  
+
   ignore_requires = TRUE;
+  ignore_requires_private = TRUE;
 
   g_hash_table_foreach (locations, max_len_foreach, &mlen);
   g_hash_table_foreach (locations, packages_foreach, GINT_TO_POINTER (mlen + 1));
@@ -1519,3 +1498,27 @@ disable_private_libs(void)
 {
   ignore_private_libs = TRUE;
 }
+
+void
+enable_requires(void)
+{
+  ignore_requires = FALSE;
+}
+
+void
+disable_requires(void)
+{
+  ignore_requires = TRUE;
+}
+
+void
+enable_requires_private(void)
+{
+  ignore_requires_private = FALSE;
+}
+
+void
+disable_requires_private(void)
+{
+  ignore_requires_private = TRUE;
+}
